#!/bin/bash

make_imgs() {
	local DIR=$1;
	if [ -s $DIR/output ]; then
		echo "\begin{verbatim}";
		awk '{print ">>> " $0}' $DIR/output;
		echo '\end{verbatim}';
	fi
	>$DIR/imgs
	local IMGS=$(find $DIR -name "*.ppm" -or -name "*.pgm" -or -name "*.png"| sort);
	if [ -n "${IMGS}" ]; then
		for IMG in $IMGS; do
			local OUT=${IMG%.*}.png;
			convert $IMG $OUT;
			echo $(basename $IMG) $OUT >> $DIR/imgs
		done;
	fi
}

#make_imgtable() {
#	local DIR=$1;
#	N=$(wc -l <$DIR/imgs);
#	TABFMT=$(for i in $(seq $N); do printf c; done);
#	echo '\begin{center}';
#	echo "\begin{tabular}{${TABFMT}}";
#	awk 'NR!=1 {print "&"} {printf "\\includegraphics[width=\\myimagewidth]{%s}",$2}' < $DIR/imgs;
#	echo '\\';
#	awk 'NR!=1 {print "&"} {printf "{\\small \\verb|%s|}",$1}' < $DIR/imgs;
#	echo '\end{tabular}';
#	echo '\end{center}';
#}

make_imgtable() {
	local DIR=$1;
	N=$(wc -l <$DIR/imgs);
	TABFMT=$(for i in $(seq $N); do printf c; done);
	echo '\begin{center}';
	echo "\begin{tabular}{${TABFMT}}";
	awk '\
		{PREF="\\\\"} \
		NR%2==0 {PREF="&"} \
		NR==1 {PREF=""} \
		{printf "%s \\parbox[t]{\\myimagewidth}{\\includegraphics[width=\\myimagewidth]{%s} \\\\ {\\tt \\small \\detokenize{%s}}}\n", \
		PREF,$2,$1} \
		' < $DIR/imgs;
	echo '\end{tabular}';
	echo '\end{center}';
}

make_IO() {
	local DIR=$1;
	printf '\\begin{verbatim}\n'
	cat $DIR/input
	printf '\\end{verbatim}\n'

	if [ -s $DIR/output ]; then
		echo "\begin{verbatim}";
		awk '{print ">>> " $0}' $DIR/output;
		echo '\end{verbatim}';
	fi
}

make_tex() {
	local DIR=$1;
	local N=0;
	if [ -s $DIR/imgs ]; then
		N=$(wc -l <$DIR/imgs);
	fi

	CODEWIDTH=0.6
	IMAGWIDTH=0.4
	if [ $N -ge 2 ]; then
		CODEWIDTH=0.5
		IMAGWIDTH=0.5
	fi
	echo "\begin{tabular}{lc}"
	echo "\begin{minipage}{${CODEWIDTH}\linewidth}"
	make_IO $DIR;
	echo '\end{minipage}'

	echo '&'

	if [ -s $DIR/imgs ]; then
		echo "\begin{minipage}{${IMAGWIDTH}\linewidth}"
		local N=$(wc -l <$DIR/imgs);
		make_imgtable $DIR;
		echo '\end{minipage}'
	fi
	echo '\\'
	echo "\end{tabular}"
}

INFILE=$1;
DIR=snippets/$(md5sum $INFILE | awk '{print $1}');

if [ ! -d "$DIR" ]; then
	mkdir -p $DIR;

	if [ -e pre.sh ]; then
		mv pre.sh $DIR;
		( cd $DIR; bash pre.sh );
	fi

	cp $INFILE $DIR/input;
	( cd $DIR; ../../../selav1d <input >output 2>&1 );

	if [ -e post.sh ]; then
		mv post.sh $DIR;
		( cd $DIR; bash post.sh );
	fi

	make_imgs $DIR;
fi

{
echo '\cprotect\fbox{'
echo '\begin{minipage}{\linewidth}'
make_tex $DIR;
echo '\end{minipage}'
echo '}'
} > $DIR/in.tex;


echo
echo
cat $DIR/in.tex;

rm -f pre.sh snip.sh post.sh >/dev/null 2>&1
