/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SLV_X_H
#define SLV_X_H

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <cell.h>
#include <psd.h>
#include <psd_eval.h>

typedef void (*fun_t)(const double*, double*, int, const void*);


class xwindow {
public:

	size_t width = 500;
	size_t height = 500;

	double limits[4] = {-1,-1,1,1};
	double* data;
	double* coords;
	double min = INFINITY;
	double max = -INFINITY;
	double rectangle[4] = {0,0,0,0};


	fun_t fun;
	void* fargs;

	SLV::psd<2>* psd = NULL;
	SLV::cell<2>* root = NULL;

	const size_t DRAWCELLS       = 1<<0;
	const size_t DRAWPSD         = 1<<1;
	const size_t DRAWGRID        = 1<<2;
	const size_t DRAWNEGATIVE    = 1<<3;
	const size_t DRAWCONTOURS    = 1<<4;
	const size_t DRAWPROJECTION  = 1<<5;
	const size_t DRAWRECTANGLE   = 1<<6;
	const size_t DRAWPOINTS      = 1<<7;
	size_t drawmask = DRAWPSD | DRAWGRID | DRAWPROJECTION;
	int colormap(double,double,double,bool);

	Display* dpy;
	Window  window;
	XSetWindowAttributes attributes;
	XGCValues gr_values;
	XFontStruct *fontinfo;
	GC gc, gc_grid, gc_text, gc_black, gc_red, gc_blue;
	Visual *visual;
	int depth;
	int screen;
	XEvent event;
	XColor color[32];
	XColor dummy;
	XClassHint* hints;

	// Image stuff
	int bitmap_pad = 32;
	int bytes_per_line = 0;
	char* image32;
	XImage* img;

	xwindow();
	~xwindow();
	void alloc();
	void dealloc();
	void set_name(const char*);
	void resize(int,int);
	void update();
	void loop();
	void center();
	void rescale_colormap();
	void draw_rectangle(double*) const;
	void draw_samplepoints(const SLV::cell<2>*) const;
	void draw_cell(const SLV::cell<2>*);
	void draw_cells(const SLV::cell<2>* cell, int depth);
	void draw_grid(int n);
	void draw_line(double,double,double,double) const;
	void draw_plot(const double* q, const double* p, int n, int h) const;
	void draw_projection() const;
	void screenshot(FILE* fname);
	void write_pgm(FILE* fname);
	void write_data(FILE* fname);
	int x(double d);
	int y(double d);

	bool b3pressed = false;
};

#endif
