#include <math.h>

// See Abramowitz & Stegun

double bessi1(const double x) {
	double xabs = fabs(x);

	double t;
	if(xabs<=3.75) { // 9.8.3
		t = x/3.75;
		t *= t;
		return x*(
				0.5
			+ t * ( 0.87890594
			+ t * ( 0.51498869
			+ t * ( 0.15084934
			+ t * ( 0.02658733
			+ t * ( 0.00301532
			+ t * ( 0.00032411
			))))))
		);
	} else { // 9.8.4
		t = 3.75/xabs;
		return (x>0?1.:-1.) * (exp(xabs) / sqrt(xabs)) * (
				0.39894228
			 + t * ( -0.03988024
			 + t * ( -0.00362018
			 + t * ( +0.00163801
			 + t * ( -0.01031555
			 + t * ( +0.02282967
			 + t * ( -0.02895312
			 + t * ( +0.01787654
			 + t * ( -0.00420059
			))))))))
		);

	}
}


double bessk1(double x) {
	double t;

	if(x<=2.0) { // 9.8.7
		t = x/2.0;
		t *= t;
		return log(x/2.)*bessi1(x)
			+ ( 1.0
			+ t * ( +0.15443133
			+ t * ( -0.67278579
			+ t * ( -0.18156897
			+ t * ( -0.01919402
			+ t * ( -0.00110404
			+ t * ( -0.00004686
			))))))
		)/x;
	} else { // 9.8.8
		t = 2.0/x;
		return (exp(-x)/sqrt(x)) * (
			+ 1.25331414
			+ t * ( +0.23498619
			+ t * ( -0.03655620
			+ t * ( +0.01504268
			+ t * ( -0.00780353
			+ t * ( +0.00325614
			+ t * ( -0.00068245
			))))))
		);
	}

}
