/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SLV_CSR_H
#define SLV_CSR_H

#include <cstddef>


struct solve_csr_args {
	double T;  // ``Sampling time''
	int npad;  // Padding factor
	double length; // magnet length
	double angle; // bending angle
	double s; // position in magnet
	double gamma; // relativisic factor
	int mode; // csr calculation mode
	int filter; // smoothing filter type
	double filterwidth; // width of the smoothing filter
	int transient; // whether to include the transient T2 term
	double large_dist_cutoff; // smallest distance up to which the large-distance limit is taken to be valid [R/gamma**3]
};

void solve_csr(const double* data, double* out, size_t npts, const void* args_, char* prefix);
void solve_csr_old(const double* data, double* out, size_t npts, const void* args_, char* prefix);

#endif
