/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

// TODO: Write equivalent functions that take an ``evaluator'' as arguments

#ifndef SLV_MATHEVAL_H
#define SLV_MATHEVAL_H

#include <string.h>
#include <matheval.h>

void* p_evaluator_create(char* fstr);

struct math_eval_args {
	char* fstr;
	char** names;
};


template <int DIM>
void eval_mathf(const double* fin, const double* zin, double* out, int npts, const void* _arg) {
	struct math_eval_args* args = (struct math_eval_args*) _arg;
	double values[1+DIM];
	void* ev = p_evaluator_create(args->fstr);
	for(int i=0; i<npts; i++,fin++,out++,zin+=DIM) {
		memcpy(values+1,zin,DIM*sizeof(double));
		values[0] = *fin;
		*out = evaluator_evaluate(ev, 1+DIM, args->names, values);
	}
	evaluator_destroy(ev);
}

template <int DIM>
void eval_math(const double* zin, double* out, int npts, const void* _arg) {
	struct math_eval_args* args = (struct math_eval_args*) _arg;
	double values[DIM];
	void* ev = p_evaluator_create(args->fstr);
	for(int i=0; i<npts; i++,out++,zin+=DIM) {
		memcpy(values,zin,DIM*sizeof(double));
		*out = evaluator_evaluate(ev, DIM, args->names, values);
	}
	evaluator_destroy(ev);
}

struct map_analytic_args {
	int dim;
	char* fstrs[64];
	char* names[64];
};

void map_analytic(const double* z, double* out, int npts, const void* args, double t);

#endif
