/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SLV_POISSON_H
#define SLV_POISSON_H

#include <complex.h>
#include <cstddef>

typedef void  (*fun_t)(const double*, double*, int, const void*);
typedef void (*cfun_t)(const double*, _Complex*, int, const void*);

void green_f_adisk(const double* in, _Complex* out, int npts, const void* args);
void green_f_csr(const double* in, _Complex* out, int npts, const void* args);

struct solve_poisson_args {
	double T; // ``Sampling time''
	int npad; // Padding factor
	fun_t  green_s; // Greens function in spatial domain
	cfun_t green_f; // Greens function in frequency domain
	void* fargs; // Arguments for greens function
};

void solve_poisson(const double* data, double* out, size_t npts, const void* args, char* prefix = NULL);

#endif
