/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SLV_PSDDOTGRAPH_H
#define SLV_PSDDOTGRAPH_H

#include "psd.h"
#include <cstdio>

namespace SLV {

template<int DIM>
void dotgraph(SLV::cell<DIM>* cell, FILE* fd, int depth=-1) {
	fprintf(fd,"digraph G {\n");
	fprintf(fd,"node[label=""];\n");
	dotgraph_worker(cell,fd,depth);
	fprintf(fd,"}");

}

template<int DIM>
void dotgraph_worker(SLV::cell<DIM>* cell, FILE* fd, int depth) {
	if(depth==0) return;
	for(int n=0; n<1<<DIM; n++) {
		if(cell->cells[n]) {
			fprintf(fd,"\"%p\" -> \"%p\" ;\n",cell,cell->cells[n]);
		}
	}
	fprintf(fd,"{\n");
	for(int n=0; n<1<<DIM; n++) {
		if(cell->cells[n]) {
			dotgraph_worker(cell->cells[n],fd,depth-1);
		}
	}
	fprintf(fd,"}\n");
}

}

#endif

