/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SLV_PSDPLOT_H
#define SLV_PSDPLOT_H

#include "psd.h"
#include "psd_projection.h"
#include <cstdio>

namespace SLV {

template<int DIM>
void psd_plot(SLV::psd<DIM>* psd, FILE* fd, double max=0.0, double min=0.0) {
	grid_t p = psd_projection(psd,0);
	//size_t NGRID = 1<<(p.nexp*p.dim);
	int N = 1<<(p.nexp);

	char header[512];
	//sprintf(header,"P5\n%d %d\n65535\n",N,N);
	sprintf(header,"P5\n%d %d\n255\n",N,N);
	fputs(header,fd);
	if(max==0) max = psd->max();
	for(int j=N-1; j>=0; j--) {
		for(int i=0; i<N; i++) {
			int k = i+j*N;
			//int val = 65535*(grid[k]/max);
			//fputc(val>>8 & 0xff,fd);
			//fputc(val    & 0xff,fd);
			int val = 255*(fabs(p.data[k])/max);
			fputc(val,fd);
		}
	}

	free(p.data);
	free(p.limits);
}

}

#endif
