/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SLV_PSDWRITE_H
#define SLV_PSDWRITE_H

#include <psd.h>
#include <psd_eval.h>

template<int DIM>
void psd_write_grid(SLV::psd<DIM>* psi, FILE* file, double* bbox, int* npts) {
	double limits[2*DIM];
	if(bbox==NULL) {
		for(int d=0; d<DIM; d++) {
			limits[d]     = psi->root->cpoint[d] - psi->root->cwidth[d];
			limits[d+DIM] = psi->root->cpoint[d] + psi->root->cwidth[d];
		};
		bbox=limits;
	}
	int nx = npts[0];
	int ny = npts[1];
	double dq=(bbox[2]-bbox[0])/(nx-1);
	double dp=(bbox[3]-bbox[1])/(ny-1);
	double z[2];
	for(int i=0; i<nx; i++) {
		for(int j=0; j<ny; j++) {
			z[0]=bbox[0]+i*dq;
			z[1]=bbox[1]+j*dp;
			double val;
			SLV::psd_eval<DIM>(z,&val,1,psi);
			fprintf(file,"%.16e %.16e %.16e\n", z[0], z[1], val);
		}
		fprintf(file,"\n");
	}
}

//void write_slice(FILE* file, coord_type pt, int axis, int npts) {
//	double d = root->cwidth[axis]*2.0/(npts-1);
//	double x0 = root->cpoint[axis]-root->cwidth[axis];
//	double val = 0.0;
//	double x = 0.0;
//	for(int i=0; i<npts; i++) {
//		x = x0+i*d;
//		pt[axis] = x;
//		val = operator()(pt);
//		fprintf(file,"%e %e\n",x,val);
//	}
//}

#endif
