/*
* libselav - a framework for semi-lagrangian simulation of sparse densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <cstdlib>
#include <math.h>

#include "read_ensembles.h"


struct ensemble_t read_ensemble(FILE* fd) {
	int size = 1000;
	int n=0;
	double* tmp = (double*) malloc(sizeof(double)*size);
	while(fscanf(fd,"%le",tmp+n) != EOF) {
		if(n==size-1) {
			size *= 2;
			tmp = (double*) realloc(tmp, sizeof(double)*size);
		}
		n++;
	}

	struct ensemble_t res;
	res.data = tmp;
	res.N = n/2;
	res.reference[0] = 0;
	res.reference[1] = 0;
	res.weight = 1; 
	return res;
}

struct ensemble_t read_astra(FILE* fd) {
	double x,y,z,px,py,pz,clock,charge,index,flag;
	fscanf(fd,"%le %le %le %le %le %le %le %le %le %le",
	&x, &y, &z, &px, &py, &pz, &clock, &charge, &index, &flag); // Reference prtcl
	double E = pz;
	double macro_charge=charge;
	int size = 1000;
	int n=0;
	double* tmp = (double*) malloc(sizeof(double)*size);
	while( 	fscanf(fd,"%le %le %le %le %le %le %le %le %le %le",
	&x, &y, &z, &px, &py, &pz, &clock, &charge, &index, &flag) != EOF) {
		if(size<=2*n+2) { size *= 2; tmp = (double*) realloc(tmp, sizeof(double)*size); }
		tmp[2*n+0] = z;
		tmp[2*n+1] = pz;
		n++;
	}

	struct ensemble_t res;
	res.data = tmp;
	res.N = n;
	res.reference[0] = 0;
	res.reference[1] = E;
	res.weight = fabs((n+1)*macro_charge)*1e-9; 
	return res;
}
