/*
* selav1d - semi-lagrangian simulation of sparse phase-space densities
* Copyright (C) 2020-2022  Philipp Amstutz (DESY)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#define MAX_BUFFER (32)

#include <error.h>

struct buf_t {
	char* name;
	size_t lineno;
	YY_BUFFER_STATE buf;
};

struct buf_t buffer_stack[MAX_BUFFER];
int buffer_stack_ptr = -1;


void buffer_stack_push_file(const char* fname) {
	buffer_stack[buffer_stack_ptr].lineno = yylineno;
	yylineno=1;
	buffer_stack_ptr++;
	if(buffer_stack_ptr >= MAX_BUFFER)
		error(1,0,"Too many nested buffers\n");
	struct buf_t* buf = &buffer_stack[buffer_stack_ptr];
	FILE* fd = fopen(fname,"r");
	if(fd==NULL) { perror(fname); exit(1); };

	buf->name = strdup(fname);
	buf->buf = yy_create_buffer( fd, YY_BUF_SIZE );
	yy_switch_to_buffer(buf->buf);
}

void buffer_stack_push_string(const char* string) {
	buffer_stack[buffer_stack_ptr].lineno = yylineno;
	yylineno=1;
	buffer_stack_ptr++;
	if(buffer_stack_ptr >= MAX_BUFFER)
		error(1,0,"Too many nested buffers\n");
	struct buf_t* buf = &buffer_stack[buffer_stack_ptr];

	buf->name = strdup(string);
	buf->buf = yy_scan_string(string);
}

void buffer_stack_pop() {
	struct buf_t* buf = &buffer_stack[buffer_stack_ptr];
	yy_delete_buffer(buf->buf);
	free(buf->name);

	if(buffer_stack_ptr-- >= 0) {
		buf = &buffer_stack[buffer_stack_ptr];
		yylineno = buf->lineno;
		yy_switch_to_buffer(buf->buf);
	}
}

